﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Data.SqlClient;
using System.Runtime.InteropServices;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text.RegularExpressions;
using MySql.Data.MySqlClient;


namespace RmxosGUI
{
    partial class RMX_OS_GUI
    {
        private void start_server_connection()
        {
            System.Threading.Thread.Sleep(2000);
            try
            {
                // Create the socket instance
                server_clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

                // Cet the remote IP address
                IPAddress ip = IPAddress.Parse(server_hostTB.Text);
                int iPortNo = System.Convert.ToInt32(server_portTP.Text);
                // Create the end point 
                IPEndPoint ipEnd = new IPEndPoint(ip, iPortNo);
                // Connect to the remote host
                server_clientSocket.Connect(ipEnd);
                if (server_clientSocket.Connected)
                {
                    //Wait for data asynchronously 
                    WaitForData();
                }
                send_data_to_server("GUI");
            }
            catch (SocketException se)
            {
                string str;
                str = "\nConnection failed, is the server running?\n" + se.Message;
                MessageBox.Show(str);
            }

        }

        private void WaitForData()
        {
            try
            {
                if (server_pfnCallBack == null)
                {
                    server_pfnCallBack = new AsyncCallback(OnDataReceived);
                }
                SocketPacket theSocPkt = new SocketPacket();
                theSocPkt.thisSocket = server_clientSocket;
                // Start listening to the data asynchronously
                server_result = server_clientSocket.BeginReceive(theSocPkt.dataBuffer,
                                                        0, theSocPkt.dataBuffer.Length,
                                                        SocketFlags.None,
                                                        server_pfnCallBack,
                                                        theSocPkt);
            }
            catch (SocketException se)
            {
                MessageBox.Show(se.Message);
            }

        }

        private void OnDataReceived(IAsyncResult asyn)
        {
            try
            {
                SocketPacket theSockId = (SocketPacket)asyn.AsyncState;
                int iRx = theSockId.thisSocket.EndReceive(asyn);
                char[] chars = new char[iRx + 1];
                System.Text.Decoder d = System.Text.Encoding.UTF8.GetDecoder();
                int charLen = d.GetChars(theSockId.dataBuffer, 0, iRx, chars, 0);
                System.String szData = new System.String(chars);
                update_serverbox(szData);
                WaitForData();
            }
            catch (ObjectDisposedException)
            {
                System.Diagnostics.Debugger.Log(0, "1", "\nOnDataReceived: Socket has been closed\n");
            }
            catch (SocketException se)
            {
                MessageBox.Show(se.Message);
            }
        }

        private void update_serverbox(string data)
        {
            if (this.server_info_textbox.InvokeRequired)
            {
                // It's on a different thread, so use Invoke.
                SetTextCallback d = new SetTextCallback(setServerBoxText);
                this.Invoke
                    (d, new object[] { this.server_info_textbox.Text + data });
            }
            else
            {
                // It's on the same thread, no need for Invoke
                this.server_info_textbox.Text = this.server_info_textbox.Text + data;
            }

        }

        private void setServerBoxText(string text)
        {
            server_info_textbox.Text = text;
        }

        private void close_server_connection()
        {
            if (server_clientSocket != null)
            {
                server_clientSocket.Close();
                server_clientSocket = null;
            }
        }

        private void send_data_to_server(string data)
        {
            try
            {
                Object objData = data + "\n";
                byte[] byData = System.Text.Encoding.UTF8.GetBytes(objData.ToString());
                if (server_clientSocket != null)
                {
                    server_clientSocket.Send(byData);
                }
            }
            catch (SocketException se)
            {
                MessageBox.Show(se.Message);
            }
        }

    }
}
