﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Data.SqlClient;
using System.Runtime.InteropServices;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Text.RegularExpressions;
using MySql.Data.MySqlClient;


namespace RmxosGUI
{
    public partial class RMX_OS_GUI : Form
    {
        delegate void SetTextCallback(string text);
        string error_log_filepath = Application.StartupPath + "\\logs\\errors.log";
        string message_log_filepath = Application.StartupPath + "\\logs\\messages.log";
        string action_log_filepath = Application.StartupPath + "\\logs\\actions.log";
        string filePath = Application.StartupPath + "\\RMX-OS.rb";
        string batpath = Application.StartupPath + "\\startbat.bat";
        bool serverFileTested = false;
        bool serverFileState = false;
        bool serverRunState;
        bool runingintaskbar = false;
        string show_server_window = "true";
        string passwordsalt = "XS";
        string useing_GUI_addon = "false";
        string cfgPath = Application.StartupPath + "\\gui_cfg.ini";
        string rmxoscfgpath = Application.StartupPath + "\\cfg.ini";
        bool debugmodejustchanged = false;
        string helpfile = "RMX-OS Manual.chm";
        bool _cfgsaved = false;

        byte[] server_dataBuffer = new byte[10];
        IAsyncResult server_result;
        public AsyncCallback server_pfnCallBack;
        public Socket server_clientSocket;

        string NAME;
        string HOST;
        string PORT;
        string RMXOS_VERSION;

        string LOG_MESSAGES;
        string LOG_ERRORS;
        string LOG_ACTIONS;

        string DEBUG_MODE;

        string MAXIMUM_CONNECTIONS;
        string LOGIN_TIMEOUT;
        string AUTO_RESTART;
        string RESTART_TIME;
        string USE_IP_BANNING;
        string PREVENT_ZOMBIE_CLIENTS;
        string RUBY_PROMPT;
        string OPTIMIZE_DATABASE_ON_STARTUP;
        string DATABASE_CONNECTION_TIMEOUT;

        string SQL_HOSTNAME;
        string SQL_USERNAME;
        string SQL_PASSWORD;
        string SQL_DATABASE;

        string INBOX_SIZE;
        string[] EXTENSIONS = new string[] { "" };
        List<string> Extensions_list = new List<string>();

        int[] useridtable = new int[1];
        List<int> useridtable_list = new List<int>();  
        string[] usernametable = new string[1];
        List<string> usernametable_list = new List<string>();
        string[] userpasswordtable = new string[1];
        List<string> userpasswordtable_list = new List<string>();
        int[] usergrouptable = new int[1];
        List<int> usergrouptable_list = new List<int>();
        int[] userbannedtable = new int[1];
        List<int> userbannedtable_list = new List<int>();
        int usertablerows;
        int[] listindextable = new int[1];

        bool bannedchanged = false;
        int banstate;
        bool userlistloaded = false;
        

        public RMX_OS_GUI()
        {
            InitializeComponent();
            load_ini_data();
            load_data(rmxoscfgpath);
            set_data();
            cfgsaved = true;
            test_server_file();
            serverRunStateLabel.Text = "Server not running";
            serverRunStateLabel.ForeColor = Color.Black;
            serverRunState = false;
            this.Text = "RMX-OS GUI - " + NAME;
            load_log_files();
        }

        public bool cfgsaved
        {
            set 
            { 
                this._cfgsaved = value;
                if (_cfgsaved)
                {
                    savedLB.Text = "Saved";
                    savedLB.ForeColor = Color.Green;
                }
                else
                {
                    savedLB.Text = "Not Saved";
                    savedLB.ForeColor = Color.Red;
                }

            }
            get { return this._cfgsaved; }
        }

        private void load_ini_data()
        {
            IniFile cfgfile = new IniFile(cfgPath);
            if (cfgfile.IniReadValue("GUI", "SHOW_SERVER_WINDOW") == "")
            {
                cfgfile.IniWriteValue("GUI", "SHOW_SERVER_WINDOW", show_server_window);
            }
            show_server_window = cfgfile.IniReadValue("GUI", "SHOW_SERVER_WINDOW");
            if (show_server_window == "true")
            {
                showServerWindowToolStripMenuItem.Image = RmxosGUI.Properties.Resources.green_dot;
            }
            else
            {
                showServerWindowToolStripMenuItem.Image = null;
            }
            if (cfgfile.IniReadValue("GUI", "PASSWORDSALT") == "")
            {
                cfgfile.IniWriteValue("GUI", "PASSWORDSALT", passwordsalt);
            }
            passwordsalt  = cfgfile.IniReadValue("GUI", "PASSWORDSALT");
            //if (cfgfile.IniReadValue("GUI", "USEING_GUI_ADDON") == "")
            // {
            //    cfgfile.IniWriteValue("GUI", "USEING_GUI_ADDON", useing_GUI_addon);
            // }
            //useing_GUI_addon = cfgfile.IniReadValue("GUI", "USEING_GUI_ADDON");
            //if (useing_GUI_addon == "true")
            //{
            //    useingGUIAddOnToolStripMenuItem.Image = RmxosGUI.Properties.Resources.green_dot;
            //}
            //else
            //{
            // useingGUIAddOnToolStripMenuItem.Image = null;
            //}
        }

        private void load_log_files()
        {
            if (File.Exists(message_log_filepath))
            {
                TextReader rmxosLog_messages = new StreamReader(message_log_filepath);
                string message_log = rmxosLog_messages.ReadToEnd();
                message_logTB.Text = message_log;
                rmxosLog_messages.Close();
            }
            else
            {
                message_logTB.Text = "No log file";
            }
            if (File.Exists(error_log_filepath))
            {
                TextReader rmxosLog_errors = new StreamReader(error_log_filepath);
                string error_log = rmxosLog_errors.ReadToEnd();
                error_logTB.Text = error_log;
                rmxosLog_errors.Close();
            }
            else
            {
                error_logTB.Text = "No log file";
            }
            if (File.Exists(action_log_filepath))
            {
                TextReader rmxosLog_actions = new StreamReader(action_log_filepath);
                string action_log = rmxosLog_actions.ReadToEnd();
                action_logTB.Text = action_log;
                rmxosLog_actions.Close();
            }
            else
            {
                action_logTB.Text = "No log file";
            }
        }

        private void save_data(string save_path)
        {
            File.Delete(save_path);
            FileStream outStream = File.Create(save_path);
            StreamWriter rmxoscfg = new StreamWriter(outStream);
            rmxoscfg.WriteLine("NAME = '" + NAME + "'");
            rmxoscfg.WriteLine("HOST = '" + HOST + "'");
            rmxoscfg.WriteLine("PORT = " + PORT);
            rmxoscfg.WriteLine("RMXOS_VERSION = " + RMXOS_VERSION);
            rmxoscfg.WriteLine("");
            rmxoscfg.WriteLine("LOG_MESSAGES = " + LOG_MESSAGES);
            rmxoscfg.WriteLine("LOG_ERRORS = " + LOG_ERRORS);
            rmxoscfg.WriteLine("LOG_ACTIONS = " + LOG_ACTIONS);
            rmxoscfg.WriteLine("");
            rmxoscfg.WriteLine("DEBUG_MODE = " + DEBUG_MODE);
            rmxoscfg.WriteLine("");
            rmxoscfg.WriteLine("MAXIMUM_CONNECTIONS = " + MAXIMUM_CONNECTIONS);
            rmxoscfg.WriteLine("LOGIN_TIMEOUT = " + LOGIN_TIMEOUT);
            rmxoscfg.WriteLine("AUTO_RESTART = " + AUTO_RESTART);
            rmxoscfg.WriteLine("RESTART_TIME = " + RESTART_TIME);
            rmxoscfg.WriteLine("USE_IP_BANNING = " + USE_IP_BANNING);
            rmxoscfg.WriteLine("PREVENT_ZOMBIE_CLIENTS = " + PREVENT_ZOMBIE_CLIENTS);
            rmxoscfg.WriteLine("RUBY_PROMPT = " + RUBY_PROMPT);
            rmxoscfg.WriteLine("OPTIMIZE_DATABASE_ON_STARTUP = " + OPTIMIZE_DATABASE_ON_STARTUP);
            rmxoscfg.WriteLine("DATABASE_CONNECTION_TIMEOUT = " + DATABASE_CONNECTION_TIMEOUT);
            rmxoscfg.WriteLine("");
            rmxoscfg.WriteLine("SQL_HOSTNAME = '" + SQL_HOSTNAME + "'");
            rmxoscfg.WriteLine("SQL_USERNAME = '" + SQL_USERNAME + "'");
            rmxoscfg.WriteLine("SQL_PASSWORD = '" + SQL_PASSWORD + "'");
            rmxoscfg.WriteLine("SQL_DATABASE = '" + SQL_DATABASE + "'");
            rmxoscfg.WriteLine("");
            rmxoscfg.WriteLine("INBOX_SIZE = " + INBOX_SIZE);
            rmxoscfg.WriteLine("");
            rmxoscfg.WriteLine("EXTENSIONS = [");
            int extension_write_int = 0;
            for (int i = 0; i < (EXTENSIONS.Length); i++)
            {
                extension_write_int += 1;
                if ((extension_write_int - (EXTENSIONS.Length)) == 0)
                {
                    rmxoscfg.WriteLine("'" + EXTENSIONS[i] + "'");
                }
                else
                {
                    rmxoscfg.WriteLine("'" + EXTENSIONS[i] + "',");
                }
            }
            rmxoscfg.WriteLine("]");
            rmxoscfg.Close();



        }

        private void set_data()
        {
            mysqlserverAddressTB.Text = SQL_HOSTNAME;
            mysqlusernameTB.Text = SQL_USERNAME;
            mysqlpasswordTB.Text = SQL_PASSWORD;
            mysqldatabaseTB.Text = SQL_DATABASE;
            server_nameTB.Text = NAME;
            server_hostTB.Text = HOST;
            server_portTP.Text = PORT;
            rmxos_versionTB.Text = RMXOS_VERSION;
            if (LOG_MESSAGES == "true")
            {
                log_messagesCB.Checked = true;
            }
            else
            {
                log_messagesCB.Checked = false;
            }
            if (LOG_ERRORS == "true")
            {
                log_errorsCB.Checked = true;
            }
            else
            {
                log_errorsCB.Checked = false;
            }
            if (LOG_ACTIONS == "true")
            {
                log_actionsCB.Checked = true;
            }
            else
            {
                log_actionsCB.Checked = false;
            }
            if (DEBUG_MODE == "true")
            {
                debugmodejustchanged = true;
                debugModeCB.Checked = true;
            }
            else
            {
                debugModeCB.Checked = false;
            }
            inboxSizeTB.Text = INBOX_SIZE;
            maxServerConnectionsTB.Text = MAXIMUM_CONNECTIONS;
            loginTimeoutTB.Text = LOGIN_TIMEOUT;
            if (AUTO_RESTART == "true")
            {
                autoRestartCB.Checked = true;
            }
            else
            {
                autoRestartCB.Checked = false;
            }
            restartTimeTB.Text = RESTART_TIME;
            if (USE_IP_BANNING == "true")
            {
                useipbanningCB.Checked = true;
            }
            else
            {
                useipbanningCB.Checked = false;
            }
            if (PREVENT_ZOMBIE_CLIENTS == "true")
            {
                zombie_clientsCB.Checked = true;
            }
            else
            {
                zombie_clientsCB.Checked = false;
            }
            if (RUBY_PROMPT == "true")
            {
                ruby_promptCB.Checked = true;
            }
            else
            {
                ruby_promptCB.Checked = false;
            }
            if (OPTIMIZE_DATABASE_ON_STARTUP == "true")
            {
                op_databaseStartupCB.Checked = true;
            }
            else
            {
                op_databaseStartupCB.Checked = false;
            }
            database_connection_timeoutTB.Text = DATABASE_CONNECTION_TIMEOUT;
            extensionsTB.Text = "";
            foreach (object extention in EXTENSIONS)
            {
                extensionsTB.Text += extention + "\r\n";
            }

        }

        private void load_data(string load_path)
        {
            if (File.Exists(load_path))
            {
                List<string> Extensions_list = new List<string>();

                TextReader rmxoscfg = new StreamReader(load_path);
                string p1 = @"(\w+)\s*=\s*(.*)\s*";
                string pq = "'(.+?)'";
                string pq2 = "\"(.+?)\"";
                Regex r = new Regex(p1);
                Regex rq = new Regex(pq);
                Regex rq2 = new Regex(pq2);
                Regex re = new Regex(pq);
                Regex re2 = new Regex(pq2);
                bool info_loaded = false;
                bool loading_extentions = false;
                while (info_loaded != true)
                {
                    string cfg = rmxoscfg.ReadLine();
                    string value = "";
                    int extension_int = 0;
                    Match m = r.Match(cfg);
                    if (m.Success)
                    {
                        Group g = m.Groups[1];
                        Group g1 = m.Groups[2];
                        CaptureCollection cc = g.Captures;
                        CaptureCollection cc1 = g1.Captures;
                        Capture c = cc[0];
                        Capture c1 = cc1[0];
                        Match mq = rq.Match(c1.ToString());
                        Match mq2 = rq2.Match(c1.ToString());

                        if (mq.Success)
                        {
                            Group gq = mq.Groups[1];
                            CaptureCollection ccq = gq.Captures;
                            Capture cq = ccq[0];
                            value = cq.ToString();
                        }
                        else if (mq2.Success)
                        {
                            Group gq = mq2.Groups[1];
                            CaptureCollection ccq = gq.Captures;
                            Capture cq = ccq[0];
                            value = cq.ToString();
                        }
                        else
                        {
                            value = c1.ToString();
                        }
                        if (value != "")
                        {
                            if (c.ToString() == "NAME")
                            {
                                NAME = value;
                            }
                            else if (c.ToString() == "HOST")
                            {
                                HOST = value;
                            }
                            else if (c.ToString() == "PORT")
                            {
                                PORT = value;
                            }
                            else if (c.ToString() == "RMXOS_VERSION")
                            {
                                RMXOS_VERSION = value;
                            }
                            else if (c.ToString() == "LOG_MESSAGES")
                            {
                                LOG_MESSAGES = value;
                            }
                            else if (c.ToString() == "LOG_ERRORS")
                            {
                                LOG_ERRORS = value;
                            }
                            else if (c.ToString() == "LOG_ACTIONS")
                            {
                                LOG_ACTIONS = value;
                            }
                            else if (c.ToString() == "DEBUG_MODE")
                            {
                                DEBUG_MODE = value;
                            }
                            else if (c.ToString() == "MAXIMUM_CONNECTIONS")
                            {
                                MAXIMUM_CONNECTIONS = value;
                            }
                            else if (c.ToString() == "LOGIN_TIMEOUT")
                            {
                                LOGIN_TIMEOUT = value;
                            }
                            else if (c.ToString() == "AUTO_RESTART")
                            {
                                AUTO_RESTART = value;
                            }
                            else if (c.ToString() == "RESTART_TIME")
                            {
                                RESTART_TIME = value;
                            }
                            else if (c.ToString() == "USE_IP_BANNING")
                            {
                                USE_IP_BANNING = value;
                            }
                            else if (c.ToString() == "PREVENT_ZOMBIE_CLIENTS")
                            {
                                PREVENT_ZOMBIE_CLIENTS = value;
                            }
                            else if (c.ToString() == "RUBY_PROMPT")
                            {
                                RUBY_PROMPT = value;
                            }
                            else if (c.ToString() == "OPTIMIZE_DATABASE_ON_STARTUP")
                            {
                                OPTIMIZE_DATABASE_ON_STARTUP = value;
                            }
                            else if (c.ToString() == "DATABASE_CONNECTION_TIMEOUT")
                            {
                                DATABASE_CONNECTION_TIMEOUT = value;
                            }
                            else if (c.ToString() == "SQL_HOSTNAME")
                            {
                                SQL_HOSTNAME = value;
                            }
                            else if (c.ToString() == "SQL_USERNAME")
                            {
                                SQL_USERNAME = value;
                            }
                            else if (c.ToString() == "SQL_PASSWORD")
                            {
                                SQL_PASSWORD = value;
                            }
                            else if (c.ToString() == "SQL_DATABASE")
                            {
                                SQL_DATABASE = value;
                            }
                            else if (c.ToString() == "INBOX_SIZE")
                            {
                                INBOX_SIZE = value;
                            }
                            else if (c.ToString() == "EXTENSIONS")
                            {
                                loading_extentions = true;
                            }
                        }
                    }

                    if (loading_extentions)
                    {

                        MatchCollection me = re.Matches(cfg);
                        MatchCollection me2 = re2.Matches(cfg);
                        foreach (Match mee in me)
                        {
                            if (mee.Success)
                            {
                                Group ge = mee.Groups[1];
                                CaptureCollection cce = ge.Captures;
                                Capture ce = cce[0];
                                Extensions_list.Add(ce.ToString());
                                extension_int += 1;
                            }
                        }
                        foreach (Match mee2 in me2)
                        {
                            if (mee2.Success)
                            {
                                Group ge2 = mee2.Groups[1];
                                CaptureCollection cce2 = ge2.Captures;
                                Capture ce2 = cce2[0];
                                Extensions_list.Add(ce2.ToString());
                                extension_int += 1;
                            }
                        }
                        if (cfg == "]")
                        {
                            info_loaded = true;
                        }
                    }
                }

                rmxoscfg.Close();
                EXTENSIONS = new string[Extensions_list.Count];
                Extensions_list.CopyTo(EXTENSIONS);
                cfgsaved = false;
            }
            else
            {
                MessageBox.Show("No cfg.ini file options set to defalt");
                NAME = "RMX-OS";
                HOST = "127.0.0.1";
                PORT = "56969";
                RMXOS_VERSION = "1";
                LOG_MESSAGES = "false";
                LOG_ERRORS = "true";
                LOG_ACTIONS = "false";
                DEBUG_MODE = "false";
                MAXIMUM_CONNECTIONS = "50";
                LOGIN_TIMEOUT = "30";
                AUTO_RESTART = "true";
                RESTART_TIME = "5";
                USE_IP_BANNING = "false";
                PREVENT_ZOMBIE_CLIENTS = "false";
                RUBY_PROMPT = "false";
                OPTIMIZE_DATABASE_ON_STARTUP = "true";
                DATABASE_CONNECTION_TIMEOUT = "60";
                SQL_HOSTNAME = "127.0.0.1";
                SQL_USERNAME = "root";
                SQL_PASSWORD = "";
                SQL_DATABASE = "rmxosdb";
                INBOX_SIZE = "10";
                cfgsaved = false;

            }



        }

        private void atempt_run_server()
        {
            if (serverFileTested == true)
            {
                if (serverFileState == true)
                {
                    if (serverRunState != true)
                    {
                        if (cfgsaved == false)
                        {
                            DialogResult result = MessageBox.Show("The current configuration has not been saved, would you like to save it now", "Configuration Not Saved", MessageBoxButtons.YesNo);
                            if (result == DialogResult.Yes)
                            {
                                save_data(rmxoscfgpath);
                            }
                            else
                            {
                                MessageBox.Show("The server will be started with it's saved configuration, not the current one");
                            }
                        }
                        RmxosProcess.EnableRaisingEvents = true;
                        RmxosProcess.StartInfo.FileName = filePath;
                        if (show_server_window == "true")
                        {
                            RmxosProcess.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Normal;
                            RmxosProcess.StartInfo.CreateNoWindow = false;
                        }
                        if (show_server_window == "false")
                        {
                            RmxosProcess.StartInfo.WindowStyle = System.Diagnostics.ProcessWindowStyle.Hidden;
                            RmxosProcess.StartInfo.CreateNoWindow = true;
                        }
                        RmxosProcess.Start();
                        serverRunState = true;
                        serverRunStateLabel.Text = "Running";
                        serverRunStateLabel.ForeColor = Color.Green;
                        startServerMenuItem.Image = RmxosGUI.Properties.Resources.green_dot;
                        stopServerMenuItem.Image = null;
                        //if (useing_GUI_addon == "true")
                        //{
                        //   start_server_connection();
                        // }
                    }
                    else
                    {
                        MessageBox.Show("The RMX-OS server is already running");
                    }
                }
                else
                {
                    MessageBox.Show("The RMX-OS.rb server file could not be" +
                        "found. Please be sure that it is in the same directory" +
                        "as the RMX-OS GUI.exe");
                }
            }
            else
            {
                MessageBox.Show("Please press the check button to search for" +
                    "the RMX_OS.rb file");
            }
        }

        private void run_server_button_Click(object sender, EventArgs e)
        {
            atempt_run_server();
        }

        private void atempt_close_server()
        {
            close_server_connection();
            if (serverFileTested == true)
            {
                if (serverFileState == true)
                {
                    if (serverRunState == true)
                    {
                        RmxosProcess.Kill();
                        RmxosProcess.Close();
                        serverRunState = false;
                        serverRunStateLabel.Text = "Server not running";
                        serverRunStateLabel.ForeColor = Color.Black;
                        startServerMenuItem.Image = null;
                        stopServerMenuItem.Image = RmxosGUI.Properties.Resources.green_dot;
                    }
                    else
                    {
                        MessageBox.Show("Server Not Running");
                    }
                }
            }
        }

        private void close_server_button_Click(object sender, EventArgs e)
        {
            atempt_close_server();
        }

        private void buttonFileCheck_Click(object sender, EventArgs e)
        {
            test_server_file();
        }

        private void test_server_file()
        {
            if (File.Exists(filePath))
            {
                serverFileState = true;
                serverFileStateLabel.Text = "Found";
                serverFileStateLabel.ForeColor = Color.Green;

            }
            else
            {
                serverFileState = false;
                serverFileStateLabel.Text = "Not Found";
                serverFileStateLabel.ForeColor = Color.Red;

            }
            serverFileTested = true;
        }

        private void RmxosProcess_Exited(object sender, EventArgs e)
        {
            close_server_connection();
            RmxosProcess.Close();
            serverRunState = false;
            serverRunStateLabel.Text = "Server not running";
            serverRunStateLabel.ForeColor = Color.Black;
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            
            try
            {
                MySqlConnection rmxosConnection = new MySqlConnection();
                rmxosConnection.ConnectionString =
                    "server=" + mysqlserverAddressTB.Text + ";"
                    + "database=" + mysqldatabaseTB.Text + ";"
                    + "uid=" + mysqlusernameTB.Text + ";"
                    + "password=" + mysqlpasswordTB.Text + ";";
                rmxosConnection.Open();
                connectStateLabel.Text = "Connection Working";
                connectStateLabel.ForeColor = Color.Green;// connection must be openned for command
                rmxosConnection.Close();
            }
            catch (Exception ex)
            {
                connectStateLabel.Text = "No Connection";
                connectStateLabel.ForeColor = Color.Red;
                MessageBox.Show(ex.Message);
            }

        }

        private void server_nameTB_TextChanged(object sender, EventArgs e)
        {
            NAME = server_nameTB.Text;
            this.Text = "RMX-OS GUI - " + NAME;
            cfgsaved = false;
        }

        private void mysqlserverAddressTB_TextChanged(object sender, EventArgs e)
        {
            SQL_HOSTNAME = mysqlserverAddressTB.Text;
            cfgsaved = false;
        }

        private void mysqlusernameTB_TextChanged(object sender, EventArgs e)
        {
            SQL_USERNAME = mysqlusernameTB.Text;
            cfgsaved = false;
        }

        private void mysqlpasswordTB_TextChanged(object sender, EventArgs e)
        {
            SQL_PASSWORD = mysqlpasswordTB.Text;
            cfgsaved = false;
        }

        private void mysqldatabaseTB_TextChanged(object sender, EventArgs e)
        {
            SQL_DATABASE = mysqldatabaseTB.Text;
            cfgsaved = false;
        }

        private void server_hostTB_TextChanged(object sender, EventArgs e)
        {
            HOST = server_hostTB.Text;
            cfgsaved = false;
        }

        private void server_portTP_TextChanged(object sender, EventArgs e)
        {
            PORT = server_portTP.Text;
            cfgsaved = false;
        }

        private void rmxos_versionTB_TextChanged(object sender, EventArgs e)
        {
            RMXOS_VERSION = rmxos_versionTB.Text;
            cfgsaved = false;
        }

        private void log_messagesCB_CheckedChanged(object sender, EventArgs e)
        {
            if (log_messagesCB.Checked)
            {
                LOG_MESSAGES = "true";
            }
            else
            {
                LOG_MESSAGES = "false";
            }
            cfgsaved = false;
        }

        private void log_errorsCB_CheckedChanged(object sender, EventArgs e)
        {
            if (log_errorsCB.Checked)
            {
                LOG_ERRORS = "true";
            }
            else
            {
                LOG_ERRORS = "false";
            }
            cfgsaved = false;

        }

        private void log_actionsCB_CheckedChanged(object sender, EventArgs e)
        {
            if (log_actionsCB.Checked)
            {
                LOG_ACTIONS = "true";
            }
            else
            {
                LOG_ACTIONS = "false";
            }
            cfgsaved = false;
        }
        private void debugModeCB_CheckedChanged(object sender, EventArgs e)
        {
            if (debugmodejustchanged == true)
            {
                debugmodejustchanged = false;
            }
            else
            {

                DialogResult result = MessageBox.Show("Changing this setting requires that the Configuration file be saved, would you like to save it now?", "Debug Mode", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    if (debugModeCB.Checked)
                    {
                        DEBUG_MODE = "true";
                    }
                    else
                    {
                        DEBUG_MODE = "false";
                    }
                    save_data(rmxoscfgpath);
                }
                else
                {
                    debugmodejustchanged = true;
                    if (debugModeCB.Checked)
                    {
                        debugModeCB.Checked = false;
                    }
                    else
                    {
                        debugModeCB.Checked = true;
                    }
                }
            }

        }

        private void maxServerConnectionsTB_TextChanged(object sender, EventArgs e)
        {
            MAXIMUM_CONNECTIONS = maxServerConnectionsTB.Text;
            cfgsaved = false;
        }

        private void loginTimeoutTB_TextChanged(object sender, EventArgs e)
        {
            LOGIN_TIMEOUT = loginTimeoutTB.Text;
            cfgsaved = false;
        }

        private void restartTimeTB_TextChanged(object sender, EventArgs e)
        {
            RESTART_TIME = restartTimeTB.Text;
            cfgsaved = false;
        }
        private void useipbanningCB_CheckedChanged(object sender, EventArgs e)
        {
            if (useipbanningCB.Checked)
            {
                USE_IP_BANNING = "true";
            }
            else
            {
                USE_IP_BANNING = "false";
            }
            cfgsaved = false;
        }
        private void zombie_clientsCB_CheckedChanged(object sender, EventArgs e)
        {
            if (zombie_clientsCB.Checked)
            {
                PREVENT_ZOMBIE_CLIENTS = "true";
            }
            else
            {
                PREVENT_ZOMBIE_CLIENTS = "false";
            }
            cfgsaved = false;

        }

        private void ruby_promptCB_CheckedChanged(object sender, EventArgs e)
        {
            if (ruby_promptCB.Checked)
            {
                RUBY_PROMPT = "true";
            }
            else
            {
                RUBY_PROMPT = "false";
            }
            cfgsaved = false;

        }

        private void autoRestartCB_CheckedChanged(object sender, EventArgs e)
        {
            if (autoRestartCB.Checked)
            {
                AUTO_RESTART = "true";
            }
            else
            {
                AUTO_RESTART = "false";
            }
            cfgsaved = false;
        }

        private void op_databaseStartupCB_CheckedChanged(object sender, EventArgs e)
        {
            if (op_databaseStartupCB.Checked)
            {
                OPTIMIZE_DATABASE_ON_STARTUP = "true";
            }
            else
            {
                OPTIMIZE_DATABASE_ON_STARTUP = "false";
            }
            cfgsaved = false;
        }

        private void database_connection_timeoutTB_TextChanged(object sender, EventArgs e)
        {
            DATABASE_CONNECTION_TIMEOUT = database_connection_timeoutTB.Text;
            cfgsaved = false;
        }

        private void inboxSizeTB_TextChanged(object sender, EventArgs e)
        {
            INBOX_SIZE = inboxSizeTB.Text;
            cfgsaved = false;
        }

        private void savebutton_Click(object sender, EventArgs e)
        {
            save_data(rmxoscfgpath);
            cfgsaved = true;
        }

        private void extensionsTB_TextChanged(object sender, EventArgs e)
        {
            EXTENSIONS = Regex.Split(extensionsTB.Text, "\r\n");
            cfgsaved = false;

        }

        private void button1_Click(object sender, EventArgs e)
        {
            load_log_files();
        }

        private void startServerMenuItem_Click(object sender, EventArgs e)
        {
            atempt_run_server();


        }

        private void stopServerMenuItem_Click(object sender, EventArgs e)
        {
            atempt_close_server();

        }

        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void rmxos_closing(object sender, FormClosingEventArgs e)
        {
            if (runingintaskbar == false)
            {
                DialogResult result = MessageBox.Show("Do you want to keep running in the task bar?", "Closing", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    e.Cancel = true;
                    this.WindowState = System.Windows.Forms.FormWindowState.Minimized;
                    this.ShowInTaskbar = false;
                    runingintaskbar = true;
                }
                else
                {
                    close_server_connection();
                    if (serverRunState == true)
                    {
                        RmxosProcess.Kill();
                        RmxosProcess.Close();
                    }
                    tray_icon.Visible = false;
                    tray_icon.Dispose();

                }
            }
            else
            {
                close_server_connection();
                if (serverRunState == true)
                {
                    RmxosProcess.Kill();
                    RmxosProcess.Close();
                }
                tray_icon.Visible = false;
                tray_icon.Dispose();

            }


        }

        private void try_icon_doubleclicked(object sender, EventArgs e)
        {
            this.WindowState = System.Windows.Forms.FormWindowState.Normal;
            this.ShowInTaskbar = true;
            runingintaskbar = false;
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            this.WindowState = System.Windows.Forms.FormWindowState.Normal;
            this.ShowInTaskbar = true;
            runingintaskbar = false;
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void showServerWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            IniFile cfgfile = new IniFile(cfgPath);
            if (show_server_window == "false")
            {
                show_server_window = "true";
                showServerWindowToolStripMenuItem.Image = RmxosGUI.Properties.Resources.green_dot;
            }
            else if (show_server_window == "true")
            {
                show_server_window = "false";
                showServerWindowToolStripMenuItem.Image = null;
            }
            cfgfile.IniWriteValue("GUI", "SHOW_SERVER_WINDOW", show_server_window);

        }

        private void saveCfgAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            save_data(rmxoscfgpath);
            cfgsaved = true;
        }

        private void saveCfgAsToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            DialogResult save_result = saveFileDialog1.ShowDialog();
            save_data(saveFileDialog1.FileName);

        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult open_result = openFileDialog1.ShowDialog();
            load_data(openFileDialog1.FileName);
            set_data();
            cfgsaved = false;
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void richTextBox1_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(e.LinkText);
        }

        private void remoteToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void useingGUIAddOnToolStripMenuItem_Click(object sender, EventArgs e)
        {
            IniFile cfgfile = new IniFile(cfgPath);
            if (useing_GUI_addon == "false")
            {
                useing_GUI_addon = "true";
                useingGUIAddOnToolStripMenuItem.Image = RmxosGUI.Properties.Resources.green_dot;
            }
            else if (useing_GUI_addon == "true")
            {
                useing_GUI_addon = "false";
                useingGUIAddOnToolStripMenuItem.Image = null;
            }
            cfgfile.IniWriteValue("GUI", "USEING_GUI_ADDON", useing_GUI_addon);


        }

        private void RMX_OS_GUI_Load(object sender, EventArgs e)
        {

        }

        private void rMXOSHelpFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Help.ShowHelpIndex(this, helpfile);

        }

        private void getusersButton_Click(object sender, EventArgs e)
        {
            userListBox.Items.Clear();
            if (getusertable())
            {
                listindextable = new int[useridtable.Length]; 
                for (int i = 0; i < useridtable.Length; i++)
                {
                    userListBox.Items.Add(useridtable[i].ToString() + ":  " + usernametable[i].ToString());
                    listindextable[i] = i;
                }
            }
            else
            {
                MessageBox.Show("The user list failed to load");
            }
             
        }

        private bool getusertable()
        {
             
            try
            {
                MySqlConnection rmxosConnection = new MySqlConnection();
                rmxosConnection.ConnectionString =
                    "server=" + mysqlserverAddressTB.Text + ";"
                    + "database=" + mysqldatabaseTB.Text + ";"
                    + "uid=" + mysqlusernameTB.Text + ";"
                    + "password=" + mysqlpasswordTB.Text + ";";
                useridtable_list.Clear();
                usernametable_list.Clear();
                userpasswordtable_list.Clear();
                usergrouptable_list.Clear();
                userbannedtable_list.Clear(); 
                int progress;
                loaduserlistprogressbar.Visible = true;
                loaduserlistprogressbar.Value = 0;
                MySqlCommand command = rmxosConnection.CreateCommand();
                MySqlCommand command2 = rmxosConnection.CreateCommand();
                MySqlDataReader Reader;
                MySqlDataReader Reader2;
                command.CommandText = "SELECT * FROM users";
                command2.CommandText = "SELECT COUNT(*) FROM users";
                rmxosConnection.Open();
                Reader2 = command2.ExecuteReader();
                while (Reader2.Read())
                {
                    usertablerows = 0;
                    for (int i = 0; i < Reader2.FieldCount; i++)
                    {
                        usertablerows += Reader2.GetInt32(i);
                    }
                }
                Reader2.Close();
                Reader = command.ExecuteReader();
                progress = 0;
                while (Reader.Read())
                {
                    useridtable_list.Add(Reader.GetInt32(0));
                    usernametable_list.Add(Reader.GetString(1));
                    userpasswordtable_list.Add(Reader.GetString(2));
                    usergrouptable_list.Add(Reader.GetInt32(3));
                    userbannedtable_list.Add(Reader.GetInt32(4));
                    progress += 1;
                    loaduserlistprogressbar.Value = (progress / usertablerows) * 100;
                }
                Reader.Close();
                rmxosConnection.Close();
                loaduserlistprogressbar.Value = 100;
                loaduserlistprogressbar.Visible = false;
                useridtable = new int[useridtable_list.Count];
                useridtable_list.CopyTo(useridtable, 0);
                usernametable = new string[usernametable_list.Count];
                usernametable_list.CopyTo(usernametable, 0);
                userpasswordtable = new string[userpasswordtable_list.Count];
                userpasswordtable_list.CopyTo(userpasswordtable, 0);
                usergrouptable = new int[usergrouptable_list.Count];
                usergrouptable_list.CopyTo(usergrouptable, 0);
                userbannedtable = new int[userbannedtable_list.Count];
                userbannedtable_list.CopyTo(userbannedtable, 0);   
                userlistloaded = true;
                return (true);
            }
            catch (Exception ex)
            {
                loaduserlistprogressbar.Visible = false;
                loaduserlistprogressbar.Value = 0;
                MessageBox.Show(ex.Message);
                useridtable = new int[useridtable_list.Count];
                useridtable_list.CopyTo(useridtable);
                usernametable = new string[usernametable_list.Count];
                usernametable_list.CopyTo(usernametable);
                userpasswordtable = new string[userpasswordtable_list.Count];
                userpasswordtable_list.CopyTo(userpasswordtable);
                usergrouptable = new int[usergrouptable_list.Count];
                usergrouptable_list.CopyTo(usergrouptable);
                userbannedtable = new int[userbannedtable_list.Count];
                userbannedtable_list.CopyTo(userbannedtable);
                userlistloaded = false;
                return (false);
            }
              
        }

        private void userListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            editusernametextLB.Text = usernametable[listindextable[userListBox.SelectedIndex]];
            edituseridtextLB.Text = useridtable[listindextable[userListBox.SelectedIndex]].ToString();      
            if (userbannedtable[listindextable[userListBox.SelectedIndex]] == 1)
            {
                bannedCB.Checked = true;
            }
            else
            {
                bannedCB.Checked = false;
            }
            newusergroupUpDown.Value = usergrouptable[listindextable[userListBox.SelectedIndex]];
            newpassTB.Text = "";  
            bannedchanged = false;
            banstate = userbannedtable[listindextable[userListBox.SelectedIndex]];
        }

        private void userSearchButton_Click(object sender, EventArgs e)
        {
            if (userlistloaded)
            {
                searchuserlist();
            }
            else
            {
                MessageBox.Show("Please load userlist first");
            }
            
        }

        private void searchuserlist()
        {
            userListBox.Items.Clear();
            int[] temptable;
            listindextable = new int[1];
            string p1 = userSearchTB.Text.ToUpper();
            Regex r = new Regex(p1);
            for (int i = 0; i < useridtable.Length; i++)
            {
                string name = usernametable[i].ToUpper();
                Match m = r.Match(name);
                if (m.Success)
                {
                    userListBox.Items.Add(useridtable[i].ToString() + ":  " + usernametable[i].ToString());
                    temptable = new int[listindextable.Length];
                    listindextable.CopyTo(temptable, 0);
                    listindextable = new int[userListBox.Items.Count];
                    temptable.CopyTo(listindextable, 0);
                    listindextable[userListBox.Items.Count - 1] = i;
                }
            }
        }

        private void newuserButton_Click(object sender, EventArgs e)
        {
            string newusername = newusernameTB.Text;
            string newuserpassword = newuserpasswordTB.Text;
            string encryptedpassword = encryptpassword(newuserpassword);
            registernewuser(newusername, encryptedpassword, registerusergroupUpDown.Value.ToString());
            userListBox.Items.Clear();
            if (getusertable())
            {
                listindextable = new int[useridtable.Length];
                for (int i = 0; i < useridtable.Length; i++)
                {
                    userListBox.Items.Add(useridtable[i].ToString() + ":  " + usernametable[i].ToString());
                    listindextable[i] = i;
                }
            }
            else
            {
                MessageBox.Show("The user list faide to load");
            }
        }

        private string encryptpassword(string password)
        {
            string encryptedpassword = DigiWar.Security.Cryptography.UnixCrypt.Crypt(passwordsalt, password);
            return (encryptedpassword.Substring(2, encryptedpassword.Length - 2));
        }

        private void registernewuser(string username, string encryptedpassword, string group)
        {
           
            try
            {
                MySqlConnection rmxosConnection = new MySqlConnection();
                rmxosConnection.ConnectionString =
                    "server=" + mysqlserverAddressTB.Text + ";"
                    + "database=" + mysqldatabaseTB.Text + ";"
                    + "uid=" + mysqlusernameTB.Text + ";"
                    + "password=" + mysqlpasswordTB.Text + ";";
                MySqlCommand command = rmxosConnection.CreateCommand();
                MySqlCommand command2 = rmxosConnection.CreateCommand();
                MySqlCommand command3 = rmxosConnection.CreateCommand();
                MySqlDataReader Reader;
                command.CommandText = "SELECT COUNT(*) AS count FROM users WHERE username = '" + username + "'";
                command2.CommandText = "INSERT INTO users (username, password, usergroup) VALUES ('" + username + "', '" + encryptedpassword + "', " + group + ")";
                command3.CommandText = "COMMIT";
                rmxosConnection.Open();
                Reader = command.ExecuteReader();
                int numberofpeoplewithname = 0;
                while (Reader.Read())
                { 
                    for (int i = 0; i < Reader.FieldCount; i++)
                    {
                        numberofpeoplewithname += Reader.GetInt32(i);
                    }
                }
                Reader.Close();
                if (numberofpeoplewithname > 0)
                {
                    MessageBox.Show("There is already a user by this name");
                }
                else
                {
                    command2.ExecuteNonQuery();
                    command3.ExecuteNonQuery();
                }
                rmxosConnection.Close(); 
 
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void edituserButton_Click(object sender, EventArgs e)
        {
            if (userlistloaded)
            {
                edituserdata();
            }
            else
            {
                MessageBox.Show("Please load the user list first");
            }
        }

        private void edituserdata()
        {
            
            try
            {
                MySqlConnection rmxosConnection = new MySqlConnection();
                rmxosConnection.ConnectionString =
                    "server=" + mysqlserverAddressTB.Text + ";"
                    + "database=" + mysqldatabaseTB.Text + ";"
                    + "uid=" + mysqlusernameTB.Text + ";"
                    + "password=" + mysqlpasswordTB.Text + ";";
                MySqlCommand command = rmxosConnection.CreateCommand();
                command.CommandText = "UPDATE `users` SET `password` = '" + encryptpassword(newpassTB.Text) + "' WHERE `user_id` =" + useridtable[listindextable[userListBox.SelectedIndex]] + " LIMIT 1";
                MySqlCommand command2 = rmxosConnection.CreateCommand();
                command2.CommandText = "UPDATE `users` SET `banned` = '" + banstate.ToString() + "' WHERE `user_id` =" + useridtable[listindextable[userListBox.SelectedIndex]] + " LIMIT 1";
                MySqlCommand command3 = rmxosConnection.CreateCommand();
                command3.CommandText = "UPDATE `users` SET `usergroup` = '" + newusergroupUpDown.Value.ToString() + "' WHERE `user_id` =" + useridtable[listindextable[userListBox.SelectedIndex]] + " LIMIT 1";
                MySqlCommand command4 = rmxosConnection.CreateCommand();
                command4.CommandText = "COMMIT";

                rmxosConnection.Open();

                if (newpassCB.Checked)
                {
                    command.ExecuteNonQuery();
                }
                if (bannedchanged)
                {
                    command2.ExecuteNonQuery();

                }
                if (newusergroupCB.Checked)
                {
                    command3.ExecuteNonQuery();
                }
                if (newpassCB.Checked || bannedCB.Checked || newusergroupCB.Checked)
                {
                    command4.ExecuteNonQuery();
                }
                rmxosConnection.Close();

                userdataeditcompleate();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                userdataeditcompleate();
            }
        }

        private void userdataeditcompleate()
        {
            int tempindex = userListBox.SelectedIndex;
            userListBox.Items.Clear();
            if (getusertable())
            {
                listindextable = new int[useridtable.Length];
                for (int i = 0; i < useridtable.Length; i++)
                {
                    userListBox.Items.Add(useridtable[i].ToString() + ":  " + usernametable[i].ToString());
                    listindextable[i] = i;
                }
                bannedchanged = false;
                newpassCB.Checked = false;
                newusergroupCB.Checked = false;
                userListBox.SelectedIndex = tempindex;
            }
            else
            {
                bannedchanged = false;
                newpassCB.Checked = false;
                newusergroupCB.Checked = false;
                editusernametextLB.Text = "name";
                edituseridtextLB.Text = "id";
                bannedCB.Checked = false;
                newusergroupUpDown.Value = 0;
                newpassTB.Text = "";
                bannedchanged = false;
                banstate = 0;
                MessageBox.Show("The user list faide to load");
            }

              
        }

        private void bannedCB_CheckedChanged(object sender, EventArgs e)
        {
            bannedchanged = true;
            if (bannedCB.Checked)
            {
                banstate = 1;
            }
            else
            {
                banstate = 0;
            }
        }

        private void newusergroupCB_CheckedChanged(object sender, EventArgs e)
        {
            if (newusergroupCB.Checked)
            {
                newusergroupUpDown.Enabled = true;
            }
            else
            {
                newusergroupUpDown.Enabled = false;
            }
        }

        private void newpassCB_CheckedChanged(object sender, EventArgs e)
        {
            if (newpassCB.Checked)
            {
                newpassTB.Enabled = true;
            }
            else
            {
                newpassTB.Enabled = false;
            }
        }









    }
}