﻿using System;
using System.Collections.Generic;
using System.Text;

namespace RmxosGUI
{
    class TCPBuffer
    {
        private byte[] buffer;       
        private long BufferSize;  
        private long WriteHead;
        private long ReadHead;

        public TCPBuffer()
        {
            Flush();
        }
        public static System.Array ResizeArray(System.Array oldArray, long newSize)
        {
            long oldSize = oldArray.Length;
            System.Type elementType = oldArray.GetType().GetElementType();
            System.Array newArray = System.Array.CreateInstance(elementType, newSize);
            long preserveLength = System.Math.Min(oldSize, newSize);
            if (preserveLength > 0)
                System.Array.Copy(oldArray, newArray, preserveLength);
            return newArray;
        }
        public void PreAllocate(long nLength)
        {
            WriteHead = 0;
            ReadHead = 0;
            BufferSize = nLength - 1;
            buffer = new byte[BufferSize];
        }

        public void Allocate(long nLength)
        {
            BufferSize = BufferSize + nLength;
            buffer = (byte[])ResizeArray(buffer, BufferSize);
        }

        public void Flush()
        {
            WriteHead = 0;
            ReadHead = 0;
            BufferSize = 0;
            buffer = new byte[0];
        }

        public void WriteByte(byte nByte)
        {
            if (WriteHead > BufferSize)
            {
                Allocate(1);
            }
            buffer[WriteHead] = nByte;
            WriteHead = WriteHead + 1;
        }

        public void WriteBytes(byte[] nBytes)
        {
            long nLength;
            nLength = nBytes.GetLongLength(0);
            System.Array.Copy(nBytes, 0, buffer, WriteHead, nLength);
            WriteHead = WriteHead + nLength;
        }

        public void WriteInteger(int nInteger)
        {
            if ((WriteHead + 1) > BufferSize)
            {
                Allocate(2);
            }
            byte[] nBytes = System.BitConverter.GetBytes(nInteger);
            System.Array.Copy(nBytes, 0, buffer, WriteHead, 2);
            WriteHead = WriteHead + 2;
        }

        public void WriteLong(long nLong)
        {
            if ((WriteHead + 3) > BufferSize)
            {
                Allocate(4);
            }
            byte[] nBytes = System.BitConverter.GetBytes(nLong);
            System.Array.Copy(nBytes, 0, buffer, WriteHead, 4);
            WriteHead = WriteHead + 4;
        }

        public void WriteString(string nString)
        {
            byte[] sBytes;
            long sLength;
            sLength = nString.Length;
            sBytes = System.Text.Encoding.UTF8.GetBytes(nString);
            WriteLong(sLength);
            if (sLength <= 0)
            {
                return;
            }
            if ((WriteHead + sLength - 1) > BufferSize)
            {
                Allocate(sLength);
            }
            System.Array.Copy(sBytes, 0, buffer, WriteHead, sLength);
            WriteHead = WriteHead + sLength;
        }
        public byte ReadByte()
        {
            bool MoveReadHead = true;
            if (ReadHead > BufferSize)
            {
                return(0);
            }
            byte value = buffer[ReadHead];
            if (MoveReadHead) 
            {
                ReadHead = ReadHead + 1;
            }
            return (value);
        }
        public byte ReadByte(bool MoveReadHead)
        {
            if (ReadHead > BufferSize)
            {
                return(0);
            }
            byte value = buffer[ReadHead];
            if (MoveReadHead) 
            {
                ReadHead = ReadHead + 1;
            }
            return (value);
        }

        public byte[] ReadBytes(long nLength)
        {
            bool MoveReadHead = true;
            byte[] Data;
            if ((ReadHead + nLength - 1) > BufferSize)
            {
                byte[] tempData = { 0 };
                return (tempData);
            }
            Data = new byte[nLength];
            System.Array.Copy(buffer, ReadHead, Data, 0, nLength);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + nLength;
            }
            return (Data);
        }
        public byte[] ReadBytes(long nLength, bool MoveReadHead)
        {
            byte[] Data;
            if ((ReadHead + nLength - 1) > BufferSize)
            {
                byte[] tempData = { 0 };
                return (tempData);
            }
            Data = new byte[nLength];
            System.Array.Copy(buffer, ReadHead, Data, 0, nLength);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + nLength;
            }
            return (Data);
        }

        public int ReadInteger()
        {
            bool MoveReadHead = true;
            if ((ReadHead + 1) > BufferSize)
            {
                return (0);
            }
            byte[] Data;
            int value;
            Data = new byte[2];
            System.Array.Copy(buffer, ReadHead, Data, 0, 2);
            value = System.BitConverter.ToInt32(Data, 0);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + 2;
            }
            return (value);
        }
        public int ReadInteger(bool MoveReadHead)
        {
            if ((ReadHead + 1) > BufferSize)
            {
                return (0);
            }
            byte[] Data;
            int value;
            Data = new byte[2];
            System.Array.Copy(buffer, ReadHead, Data, 0, 2);
            value = System.BitConverter.ToInt32(Data, 0);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + 2;
            }
            return (value);
        }

        public long ReadLong()
        {
            bool MoveReadHead = true;
            if ((ReadHead + 3) > BufferSize)
            {
                return (0);
            }
            byte[] Data;
            int value;
            Data = new byte[4];
            System.Array.Copy(buffer, ReadHead, Data, 0, 4);
            value = System.BitConverter.ToInt32(Data, 0);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + 4;
            }
            return (value);
        }
        public long ReadLong(bool MoveReadHead)
        {
            if ((ReadHead + 3) > BufferSize)
            {
                return (0);
            }
            byte[] Data;
            int value;
            Data = new byte[4];
            System.Array.Copy(buffer, ReadHead, Data, 0, 4);
            value = System.BitConverter.ToInt32(Data, 0);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + 4;
            }
            return (value);
        }

        public string ReadString()
        {
            bool MoveReadHead = true;
            long sLength;
            byte[] sBytes;
            sLength = ReadLong();
            if (sLength <= 0)
            {
                return ("");
            }
            sBytes = new byte[sLength - 1];
            System.Array.Copy(buffer, ReadHead, sBytes, 0, sLength);
            string value;
            value = System.Text.Encoding.UTF8.GetString(sBytes);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + sLength;
            }
            return (value);
        }
        public string ReadString(bool MoveReadHead)
        {
            long sLength;
            byte[] sBytes;
            sLength = ReadLong();
            if (sLength <= 0)
            {
                return ("");
            }
            sBytes = new byte[sLength - 1];
            System.Array.Copy(buffer, ReadHead, sBytes, 0, sLength);
            string value;
            value = System.Text.Encoding.UTF8.GetString(sBytes);
            if (MoveReadHead)
            {
                ReadHead = ReadHead + sLength;
            }
            return (value);
        }

        public long Count()
        {
            return (buffer.LongLength);
        }

        public long Length()
        {
            return (Count() - ReadHead);
        }

        public byte[] ToArray()
        {
            return (buffer);
        }
               


    }
}
