#======================================================================
# Player
#----------------------------------------------------------------------
# Contains player related data.
#======================================================================

class Player
	
	# setting all accessible variables
	attr_accessor :user_id
	attr_accessor :username
	attr_accessor :usergroup
	attr_accessor :buddies
	attr_accessor :guildname
	attr_accessor :guildleader
	attr_accessor :guildmembers
	attr_accessor :map_id
	attr_reader   :character_name
	attr_reader   :x
	attr_reader   :y
	attr_reader   :real_x
	attr_reader   :real_y
	attr_reader   :direction
	attr_reader   :move_speed
	attr_reader   :pattern
	attr_reader   :anime_count
	
	#----------------------------------------------------------------------
	# Initialization.
	#----------------------------------------------------------------------
	def initialize(client)
		@client = client
		# database data for this user
		@user_id = -1
		@username = ''
		@usergroup = RMXOS::GROUP_PLAYER
		@buddies = []
		self.reset_guild
		# very useful when saving the transmitting map status since it's already on the server
		@map_id = 0
		@character_name = ''
		@x = 0
		@y = 0
		@real_x = 0
		@real_y = 0
		@direction = 2
		@move_speed = 4
		@pattern = 0
		@anime_count = 0
	end
	#----------------------------------------------------------------------
	# Resets guild data.
	#----------------------------------------------------------------------
	def reset_guild
		@guildname = ''
		@guildleader = ''
		@guildmembers = []
	end
	#----------------------------------------------------------------------
	# Evaluates the received data.
	# data - array of map data values
	#----------------------------------------------------------------------
	def evaluate(data)
		@character_name = data[0]
		@x = data[1]
		@y = data[2]
		@real_x = data[3]
		@real_y = data[4]
		@direction = data[5]
		@move_speed = data[6]
		@pattern = data[7]
		@anime_count = data[8]
	end
	#----------------------------------------------------------------------
	# Cets all player data.
	# Returns: Data string.
	#----------------------------------------------------------------------
	def get_all_data
		data = []
		data.push(@user_id.to_s)
		data.push(@map_id.to_s)
		data.push("'#{RMXOS.fix_string(@username)}'")
		data.push(@usergroup.to_s)
		return "#{data.join(',')},#{self.get_map_data}"
	end
	#----------------------------------------------------------------------
	# Cets player data for the current map.
	# Returns: Data string.
	#----------------------------------------------------------------------
	def get_current_data
		data = []
		data.push(@user_id.to_s)
		data.push("'#{RMXOS.fix_string(@username)}'")
		data.push(@usergroup.to_s)
		return "#{data.join(',')},#{self.get_map_data}"
	end
	#----------------------------------------------------------------------
	# Cets player map data.
	# Returns: Data string.
	#----------------------------------------------------------------------
	def get_map_data
		data = []
		data.push("'#{RMXOS.fix_string(@character_name)}'")
		data.push(@x.to_s)
		data.push(@y.to_s)
		data.push(@real_x.to_s)
		data.push(@real_y.to_s)
		data.push(@direction.to_s)
		data.push(@move_speed.to_s)
		data.push(@pattern.to_s)
		data.push(@anime_count.to_s)
		return data.join(',')
	end
	#----------------------------------------------------------------------
	# Cets buddies as list.
	# Returns: Buddies as list.
	#----------------------------------------------------------------------
	def get_buddies_list
		return (@buddies.size == 0 ? 'nil' : @buddies.map {|name| name = "'#{RMXOS.fix_string(name)}'"}.join(','))
	end
	#----------------------------------------------------------------------
	# Cets guild members as list.
	# Returns: Guild members as list.
	#----------------------------------------------------------------------
	def get_guildmembers_list
		return (@guildmembers.size == 0 ? 'nil' : @guildmembers.map {|name| name = "'#{RMXOS.fix_string(name)}'"}.join(','))
	end

end